using System;
using System.Data;
using System.Data.SqlClient;

namespace Dpaa.ModulosWeb.Datos
{
	/// <summary>
	/// Clase base de la capa de datos. Provee instanciacin y
	/// configuracin automtica del objeto SqlConnection, as
	/// como funciones de soporte.
	/// </summary>
	public abstract class ObjetoDb
	{
		protected SqlConnection conexion;
		private string cadenaDeConexion;

		/// <summary>
		/// El constructor toma una cadena de conexin e instancia
		/// una nueva conexin sobre esta cadena.
		/// </summary>
		/// <param name="nuevaCadenaDeConexion">Cadena de conexion a la BD</param>
		public ObjetoDb(string nuevaCadenaDeConexion)
		{
			cadenaDeConexion = nuevaCadenaDeConexion;
			conexion = new SqlConnection(cadenaDeConexion);
		}

		/// <summary>
		/// Propiedad protegida que muestra la cadena de conexin
		/// a las clases derivadas. Slo lectura.
		/// </summary>
		protected string CadenaDeConexion
		{
			get
			{
				return cadenaDeConexion;
			}
		}

		/// <summary>
		/// Automatiza la instanciacin de un objeto SqlCommand. Toma el nombre del
		/// procedimiento almacenado y un array de objetos IDataParameter. Construye
		/// un comando SQL usando estos parmetros. Una vez que los parmetros han
		/// sido aadidos, crea un nuevo parmetro que almacena el valor de retorno
		/// numrico del procedimiento almacenado. Retorna el nuevo comando recin
		/// construido.
		/// </summary>
		/// <param name="nombreProcAlmacenado">Nombre del procedimiento almacenado</param>
		/// <param name="parametros">Array de parmetros tipo IDataParameter</param>
		/// <returns>Nueva instancia de un objeto SqlCommand</returns>
		private SqlCommand ConstruyeComandoInt(string nombreProcAlmacenado,
			IDataParameter[] parametros)
		{
			SqlCommand comando = ConstruyeComandoQuery(nombreProcAlmacenado, parametros);
			comando.Parameters.Add(new SqlParameter("valorDeRetorno", SqlDbType.Int,
				4, ParameterDirection.ReturnValue, false, 0, 0, string.Empty,
				DataRowVersion.Default, null));
			return comando;
		}

		/// <summary>
		/// Crea un objeto comando SQL. Se usa para devolver los resultados de un
		/// SELECT en procedimientos almacenados a un DataSet o DataReader.
		/// </summary>
		/// <param name="nombreProcAlmacenado">Nombre del procedimiento almacenado</param>
		/// <param name="parametros">Array de parmetros tipo IDataParameter</param>
		/// <returns>Un comando SQL</returns>
		private SqlCommand ConstruyeComandoQuery(string nombreProcAlmacenado,
			IDataParameter[] parametros)
		{
			SqlCommand comando = new SqlCommand(nombreProcAlmacenado, conexion);
			comando.CommandType = CommandType.StoredProcedure;
			foreach(SqlParameter p in parametros)
			{
				comando.Parameters.Add(p);
			}
			return comando;
		}

		/// <summary>
		/// Ejecuta un procedimiento almacenado, facilitando la realizacin de 
		/// Inserts, Updates y Deletes y devuelve un cdigo numrico que puede
		/// ser interpretado por las clases derivadas para determinar el xito
		/// en la ejecucin o algn tipo de fallo.
		/// </summary>
		/// <param name="nombreProcAlmacenado">Nombre del procedimiento almacenado</param>
		/// <param name="parametros">Array de parmetros tipo IDataParameter</param>
		/// <param name="filasAfectadas">Nmero de filas afectadas por el procedimiento</param>
		/// <returns>Un entero que indica el valor de retorno del procedimiento</returns>
		protected int EjecutaProc(string nombreProcAlmacenado, 
			IDataParameter[] parametros, out int filasAfectadas)
		{
			int resultado;
			conexion.Open();
			SqlCommand comando = ConstruyeComandoInt(nombreProcAlmacenado, parametros);
			filasAfectadas = comando.ExecuteNonQuery();
			resultado = (int)comando.Parameters["valorDeRetorno"].Value;
			conexion.Close();
			return resultado;
		}

		/// <summary>
		/// Esta sobrecarga guarda el resultado de la ejecucin de un procedimiento
		/// almacenado en un objeto SqlDataReader
		/// </summary>
		/// <param name="nombreProcAlmacenado">Nombre del procedimiento almacenado</param>
		/// <param name="parametros">Array de parmetros tipo IDataParameter</param>
		/// <returns>Un objeto instanciado SqlDataReader</returns>
		protected SqlDataReader EjecutaProc(string nombreProcAlmacenado,
			IDataParameter[] parametros)
		{
			SqlDataReader resultado;
			conexion.Open();
			SqlCommand comando = ConstruyeComandoQuery(nombreProcAlmacenado, parametros);
			comando.CommandType = CommandType.StoredProcedure;
			resultado = comando.ExecuteReader(CommandBehavior.CloseConnection);
			return resultado;
		}

		/// <summary>
		/// Esta sobrecarga guarda el resultado en un DataSet.
		/// </summary>
		/// <param name="nombreProcAlmacenado">Nombre del procedimiento almacenado</param>
		/// <param name="parametros">Array de parmetros tipo IDataParameter</param>
		/// <param name="tabla">Nombre de la tabla</param>
		/// <returns>Un objeto DataSet</returns>
		protected DataSet EjecutaProc(string nombreProcAlmacenado,
			IDataParameter[] parametros, string tabla)
		{
			DataSet ds = new DataSet();
			conexion.Open();
			SqlDataAdapter da = new SqlDataAdapter();
			da.SelectCommand = ConstruyeComandoQuery(nombreProcAlmacenado, parametros);
			da.Fill(ds, tabla);
			conexion.Close();
			return ds;
		}

		/// <summary>
		/// Esta sobrecarga guarda el resultado en un DataSet ya existente.
		/// </summary>
		/// <param name="nombreProcAlmacenado">Nombre del procedimiento almacenado</param>
		/// <param name="parametros">Array de parmetros tipo IDataParameter</param>
		/// <param name="ds">Un objeto DataSet</param>
		/// <param name="tabla">Nombre de la tabla</param>
		protected void EjecutaProc(string nombreProcAlmacenado, 
			IDataParameter[] parametros, DataSet ds, string tabla)
		{
			conexion.Open();
			SqlDataAdapter da = new SqlDataAdapter();
			da.SelectCommand = ConstruyeComandoQuery(nombreProcAlmacenado, parametros);
			da.Fill(ds, tabla);
			conexion.Close();
		}
	}
}
